/*  $Id: AStateMachineCommand.java 1338 2012-08-25 17:38:30Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

/** 
 * Command controled by state machine.
 * Command can be run if related state machine state allows it. 
 * State machine state may change after command is done.
 */
public abstract class AStateMachineCommand<TMachineSt,TMachineEv,TEv extends Event>
		implements Runnable
	 {
	/** 
	 * Create new command.
	 * The automata checks if command can execute regarding its internal state
	 * and the transition linked to command start. The end error is determined 
	 * by apparition of error during execution. The automata's state is updated
	 * according the end transition. When same transition is given for start,
	 * succeed and fail events, no transition is applied at command end.
	 * @param stateMachine execution controler automata.
	 * @param evHandler target for all automated events created during command 
	 * execution.
	 * @param evProducer event generator to use to create events.
	 * @param errHandler error handler.
	 * @param startEv transition linked to command start.
	 * @param okEv transition linked to command succes.
	 * @param koEv transition linked to command failure.
	 */ 
	public AStateMachineCommand(
						StateMachine<TMachineSt,TMachineEv> stateMachine,
						Handler<TEv> evHandler,
						EventProducer<TEv> evProducer,
						ErrorHandler errHandler,
						TMachineEv startEv, TMachineEv okEv,
						TMachineEv koEv) 
	{
		_stateMachine = stateMachine;
		_evHandler = evHandler;
		_evProducer = evProducer;
		_errHandler = errHandler ;
		_startEv = startEv;
		_okEv = okEv;
		_koEv = koEv;
	}
	/** 
	 * Command entry point.
	 * Override this method to define command job.<br> 
	 * <b>warning:</b> don't override run() to preserv default state machine
	 * management.
	 */
	public abstract void oRun();
	/** 
	 * Runnable implementation. 
	 * <b>warning:</b> don't override run(), override oRun() instead to define the 
	 * command  job.
	 */ 
	public void run()
	{
		TEv ev;
		boolean ok=_stateMachine.runTransition(_startEv);
		if (ok)
		{
			if (!_errHandler.checkAndReset())
			{
				App.get().log().warn(
					"Err handler unchecked before commande start !");
			}
			ev=_evProducer.newEvent(Event.I_CMDSTART,
											Event.ELevel.DEBUG);
			ev.addArg(_startEv);
			_evHandler.handle(ev);
			oRun();
			ok=_errHandler.checkAndReset();
			boolean endState=true;
			if ((!_startEv.equals(_okEv)) || (!_startEv.equals(_koEv)))
			{
				// When all events are the same, no end transition.
				endState=_stateMachine.runTransition(ok?_okEv:_koEv);
			}
			ev=_evProducer.newEvent(
						ok?Event.I_CMDDONE:Event.I_CMDFAILED,
						ok?Event.ELevel.INFO:Event.ELevel.ERROR);
			ev.addArg(_startEv);
			_evHandler.handle(ev);
			if (jc.O.DEBUG) {
				if (!endState)
				{
					App.get().log().warn(
	 "Can't switch to node command terminal state. State graph may be wrong.");
				}
			}
		}
		else 
		{
			ev=_evProducer.newEvent(
				Event.I_CMDREFUSED, Event.ELevel.WARNING);
			ev.addArg(_startEv);
			_evHandler.handle(ev);
		}
	}
	/** Event to report command start. */
	private TMachineEv _startEv;
	/** Event to report command succes. */
	private TMachineEv _okEv;
	/** Event to report command failure. */
	private TMachineEv _koEv;
	/** 
	 * State machine controlling this command execution.
	 * @djin.relation aggregation
	 */ 
	private StateMachine<TMachineSt,TMachineEv> _stateMachine;
	/** 
	 * Event generator. 
	 * @djin.relation aggregation
	 */ 
	private EventProducer<TEv> _evProducer;
	/** 
	 * Error handler.
	 * @djin.relation aggregation
	 */ 
	private ErrorHandler _errHandler;
	/** 
	 * Get error handler. 
	 * Use to check if error appeared during command execution.
	 * @return the error handler.
	 */ 
	protected ErrorHandler getErrHandler()
	{
		return _errHandler;
	}
	/** 
	 * Event handler.
	 * @djin.relation aggregation
	 */ 
	private Handler<TEv> _evHandler;
}
