/*  $Id: OSCmdRunner.java 1338 2012-08-25 17:38:30Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2010 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;

/**
 * Helper for executing OS commands
 * @version $Id: OSCmdRunner.java 1338 2012-08-25 17:38:30Z sdevaux $
 * @author Sebastien Devaux
 */ 
public class OSCmdRunner implements Runnable {
	/**	
	 * create new OS command runner
	 * @param command command line (program name + args)
	 */ 
	public OSCmdRunner(String[] command) {
		_commandLine=command;
		_callback=new PrintStreamCallback(System.out);
        _directory=null;
	}
	/**
	 * Change output stream.
	 * <p>set the stream where running command outputs are rederected.</p>
	 * <p><b>warning:</b> this methode change the callback used to process command
	 * outputs.</p>
	 * @param o new output stream.
	 */ 
	public void setPrintStream(PrintStream o) {
		_callback=new PrintStreamCallback(o);
	}
	/**
	 * Change output stream callback.
	 * @param c new callback.
	 */ 
	public void setCallback(Callback c) {
		_callback=c;
	}
    /**
     * Change command working directory
     * @param dir new working directory
     */
   public void setWorkingDirectory(String dir) {
       _directory=new File(dir);
   } 
    /**
     * Change command working directory
     * @param dir new working directory
     */
    public void setWorkingDirectory(File dir) {
        _directory=dir;
    } 
    /**
	 *
	 */
    protected void processStarted() {
		try {
			BufferedReader r=new BufferedReader(
							new InputStreamReader(_process.getInputStream()));
			String line;
			do {
				line=r.readLine();
				if (line!=null) {
					_callback.processLine(line);
				}
			} while (line!=null);
		} catch (IOException ex) {
            handleError(ERR_OSTREAM,Error.ELevel.WARN,ex);
		}
    }
	/**
	 * run command
	 */ 
	public void run() {
		ProcessBuilder processBuilder=new ProcessBuilder(_commandLine);
		processBuilder.redirectErrorStream(true);
        processBuilder.directory(_directory);
		try {
			_process=processBuilder.start();
			StringBuffer msg=new StringBuffer();
			for (int i=0;i<_commandLine.length;i++) {
				msg.append(_commandLine[i]);
				msg.append(" ");
			}
			App.get().log().debug("Command execution started : "+msg);
		}
		catch (IOException ex) {
            handleError(ERR_NOTFOUND,Error.ELevel.ERR,ex);
			return;
		}	
		catch (SecurityException ex) {
            handleError(ERR_NOTALLOWED,Error.ELevel.ERR,ex);
			return;
		}	
		processStarted();
		try {
			_process.waitFor();
			App.get().log().debug("Command execution ended("+_process.exitValue()
							+") : "+_commandLine[0]);
		}
		catch (InterruptedException ex) {
            handleError(ERR_INTERRUPT,Error.ELevel.WARN,ex);
		}
	}
	/**
	 * Runs OS command in separate thread.
	 * Outputs stdout and stderr of running command to System.out.
	 * @param cmd command line (program name + args)
	 */
	public static OSCmdRunner exec(String[] cmd) {	
		OSCmdRunner oscmd=new OSCmdRunner(cmd);
		EThread t=new EThread(oscmd.getClass().getName()+":"+cmd[0],oscmd);
        oscmd._thread=t;
		t.start();
		return oscmd;
	}
    /** thread used when desynchronized command run */
    private EThread _thread=null;
    /**
     * Wait for command completion.
     * Does nothing when command not runned with its own thread.
     */
    public void join() {
        join(0);
    }
    /**
     * Wait for command completion.
     * Does nothing when command not runned with its own thread.
     * @param timeout max wait delay
     */
    public void join(long timeout) {
        if (_thread!=null) {
            try {
                _thread.join(timeout);
            }
            catch (InterruptedException ex) {
                handleError(ERR_INTERRUPT,Error.ELevel.WARN,ex);
            }
        }
    }
    /**
     *
     */
	public static OSCmdRunner exec(String cmd) {
		StringTokenizer st=new StringTokenizer(cmd);
		return exec(st.split());
	}
	/**
	 * @return command exit value when available
	 */
	public int getExitValue() {
		return _process.exitValue();
	}
	/**
	 * @return process
	 */
	protected Process getProcess() {
		return _process;
	}	
	/** system process for command execution */	
	private Process _process;
	/** Command line arguments */
	private String[] _commandLine;
    /**
     * @return command line
     */
    protected String[] getCommandLine() {
        return _commandLine;
    }
	/** command output stream handler */
	private Callback _callback;
    /** command working directory */
    private File _directory;
    /** error handler */
    private ErrorHandler _errHandler;
    /** 
     * handle error
     * @param errId on of ERR_* consts
     * @param level one of Error.L_* consts
     * @param ex exception source of error (null if not any)
     */
    private void handleError(String errId, Error.ELevel level, Exception ex) {
        if (_errHandler==null) {
            _errHandler=App.get().getErrHandler();
        }
        Error err=new Error(errId,level,ex);
        err.addArg(_commandLine[0]);
        _errHandler.handle(err);
    }
    /**
     * Get command error handler.
     * The one set with setErrorHandler(), Default App error
     * handler if not any.
     * @return error handler
     */
    protected ErrorHandler getErrorHandler() {
        if (_errHandler==null) {
            _errHandler=App.get().getErrHandler();
        }
        return _errHandler;
    }
    /**
     * Change error handler.
     * Error handler should be set before running command.
     * @param handler new error handler
     */
    public void setErrorHandler(ErrorHandler handler) {
        _errHandler=handler;
    } 
    // error ids
    private final static String ERR_OSTREAM="OSCmdRunner.OSTREAM";
    private final static String ERR_NOTFOUND="OSCmdRunner.NOTFOUND";
    private final static String ERR_NOTALLOWED="OSCmdRunner.NOTALLOWED";
    private final static String ERR_INTERRUPT="OSCmdRunner.INTERRUPT";

	public static interface Callback {
		public void processLine(String line);
	}
	public static class PrintStreamCallback implements Callback {
		public PrintStreamCallback(PrintStream out) {
			_output=out;
		}
		public void processLine(String line) {
			_output.println(line);
		}
		private PrintStream _output;
	}
}

// vim:tabstop=4:
