/*  $Id: ChainedSaxWriter.java 1040 2010-06-15 13:36:52Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2009 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.xml;

import java.io.IOException;
import java.io.OutputStream;
import org.xml.sax.SAXException;

/**
 * Chain same writings to a couple of SaxWriters.
 * To writes same things to two writers, itself and a chained one.
 * Can be use to spy regular SaxWriters...
 * <code>
 *	SaxWriter writer=new SaxWriter(output);
 *	writer.startElement(...);
 *	...
 *	</code>
 *	To debug it, simply add one lines :
 * <code>
 *	SaxWriter writer=new SaxWriter(output);
 * writer=new ChainedSaxWriter(System.out,writer);
 * // then all writings are sent to both writers: stdout one and original one.
 *	writer.startElement(...);
 *	...
 *	</code>
 * @version $Id: ChainedSaxWriter.java 1040 2010-06-15 13:36:52Z sdevaux $
 * @author Sebastien Devaux	
 */
public class ChainedSaxWriter extends SaxWriter {
	/**
	 * Create new chained writer
	 * @param output output stream of main writer
	 * @param cw chained writer.
	 */ 
	public ChainedSaxWriter(OutputStream output,SaxWriter cw) {
		super(output);
		_chainedWriter=cw;
	}
	/**
	 * Start new element.
	 * Previously added attributes will be set to this new elements
	 * then current attribute list is cleared.
	 * @param name new element name.
	 */ 
	public void startElement(String name) throws SAXException {
		super.startElement(name);
		_chainedWriter.startElement(name);
	}
	/**
	 * Add attribute to the next start element.
	 * @param name attribute name
	 * @param value attribute value
	 */ 
	public void addAttribute(String name,String value) {
		super.addAttribute(name,value);
		_chainedWriter.addAttribute(name,value);
	}
	/** 
	 * Add text to current element
	 * @param text text to add.
	 */
	public void addText(String text) throws SAXException {
		super.addText(text);
		_chainedWriter.addText(text);
	}
	/**
	 * Close current element
	 */ 
	public void endElement() throws SAXException {
		super.endElement();
		_chainedWriter.endElement();
	}

	public void flush() throws IOException {
		super.flush();
		_chainedWriter.flush();
	}
	/** chained writer */
	private SaxWriter _chainedWriter;
}
